/*
	LUSScreenSaver
	Copyright (C) 2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "LUSScreenSaverMisc.h"
#include "LUSScreenSaver.h"
#include "LUSScreenSaverInterface.h"
#include "LUSScreenSaverAuthInterface.h"
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

using namespace LUSMisc;

LUSScreenSaver::LUSScreenSaver()
:QObject()
{
	DBG_LOG( "LUSScreenSaver::LUSScreenSaver()" );
	m_pSaver = NULL;
	m_pSaverWidget = NULL;
	m_pAuthDlg = NULL;
	m_nDisableSuspend = 200;
	m_nDisableSuspendPre = 200;
	m_bForceExecuted = FALSE;
	m_nTempMode = LUS_NOSAVE;
	m_bWaitingLauncher = FALSE;
	srand( ( unsigned int ) time( NULL ) );
}

LUSScreenSaver::~LUSScreenSaver()
{
	DBG_LOG( "LUSScreenSaver::~LUSScreenSaver()" );
	m_nTempMode = LUS_NOSAVE;
	restore();
	AuthDel();
	ResetIntervals();
}

void LUSScreenSaver::restore()
{
	DBG_LOG( "LUSScreenSaver::restore()" );
	if ( m_bForceExecuted )
	{
		DBG_LOG( "LUSScreenSaver::restore() > Nothing to do" );
		return;
	}

	SetLightLevel( GetLightLevel() );
	
	if ( m_pSaver )
	{
		LUSScreenSaverInterface* iface;
		if ( m_pSaver->queryInterface( IID_LUSScreenSaver, (QUnknownInterface**)&iface ) == QS_OK )
		{
			iface->Stop();
			iface->release();
		}
		else
		{
			ExecOKDlg( "FATAL : Saver Can't Stop" );
		}
		m_pSaver->unload();
		delete m_pSaver;
		m_pSaver = NULL;
		m_pSaverWidget = NULL;
	
		if ( ! m_pAuthDlg && IsLUSAuthInstalled() && IsPasswordEnable() )
		{
			DBG_LOG( "LUSScreenSaver::restore() > Do Authentication" );
			LUSScreenSaverAuthInterface* iface_auth;
			m_pAuthDlg = new QLibrary( "lusauth", QLibrary::Immediately );
			if ( m_pAuthDlg->queryInterface( IID_LUSScreenSaver,
											(QUnknownInterface**)&iface_auth ) == QS_OK )
			{
				iface_auth->Start( rand() );
			}
			else
			{
				DBG_LOG( "LUSScreenSaver::restore() > LUSAuth quetyInterface() failed" );
				delete m_pAuthDlg;
				m_pAuthDlg = NULL;
			}
		}
		else
		{
			RestartKeyHelper();
		}
	}

	m_nTempMode = LUS_NOSAVE;
}

bool LUSScreenSaver::save( int level )
{
	DBG_LOG( QString( "LUSScreenSaver::save( level : %1 )" ).arg( level ).latin1() );

	bool bRet = false;

	if ( m_bForceExecuted  )
	{
		DBG_LOG( "save() > Forced Mode" );
		{
			ResetIntervals();
		}
		DBG_LOG( "save() > Do ScreenSaver" );
		m_bWaitingLauncher = TRUE;
		Global::execute( "launcher" );
//		QTimer::singleShot( 10, this, SLOT( ShowScreenSaver() ) );
		ShowScreenSaver();
		bRet = TRUE;
		m_nTempMode = LUS_SCREENSAVING;
		m_bForceExecuted = FALSE;
		return bRet;
	}

	switch( level )
	{
	case 0:
		{
			if ( ( m_nDisableSuspend >= 0
				&& m_nDisableSuspend <= 0 )
				|| 
				IsSpecialAppExists( LUS_SCREENSAVING ) )
			{
				DBG_LOG( "save() > LEVEL0 intercepted" );
				bRet = true;
				break;
			}
			if ( WhichIsFirst( LUS_SCREENSAVING, LUS_LIGHTOFF, LUS_SUSPEND )
					== LUS_SCREENSAVING )
			{
				if ( IsACOnline() )
				{
					DBG_LOG( "save() > Do ScreenSaver" );
#if 1 // FIND Full Screen APP
#if 0
					bool bAnotherFullScreen = false;
				//	QList<QWSInternalWindowInfo>* pList = QWSServer::windowList();
					QWidgetList* pList = QApplication::allWidgets();
					for ( unsigned int n = 0; n < pList->count(); n++ )
					{
						// QWidget* aWin = QWidget::find( pList->at(n)->winid );
						QWidget* aWin = pList->at(n);
						if ( aWin != NULL )
						{
							DBG_LOG( QString( "save() > Widget Status" ).latin1() );
							DBG_LOG( QString( "save() >   Desktop :  %1" ).arg(aWin->isDesktop()?"TRUE":"FALSE").latin1() );
							DBG_LOG( QString( "save() >  TopLevel :  %1" ).arg(aWin->isTopLevel()?"TRUE":"FALSE").latin1() );
							DBG_LOG( QString( "save() >   Visible :  %1" ).arg(aWin->isVisible()?"TRUE":"FALSE").latin1() );
							DBG_LOG( QString( "save() >   Enabled :  %1" ).arg(aWin->isEnabled()?"TRUE":"FALSE").latin1() );
							DBG_LOG( QString( "save() >    height :  %1" ).arg(aWin->height()).latin1() );
							DBG_LOG( QString( "save() >  focus widget height :  %1" ).arg(aWin->focusWidget()->height()).latin1() );
						//	if ( aWin->isDesktop() ) continue;
							if ( aWin->isTopLevel() )
							{
								if ( ! aWin->isDesktop()
									&& aWin->isVisible()
									&& aWin->isEnabled()
									&& ( aWin->height() == QPEApplication::desktop()->height() 
									|| aWin->focusWidget()->height() == QPEApplication::desktop()->height() ) )
								{
									bAnotherFullScreen = true;
								}
								QObjectList* pObList = aWin->queryList();
								QObjectListIt it( *pObList );
								QObject * obj = NULL;
								while ( (obj=it.current()) != 0 )
								{
									++it;
									if ( obj->isWidgetType()
										&& ((QWidget*)obj)->isVisible()
										&& ((QWidget*)obj)->isEnabled()
										&& ((QWidget*)obj)->height() == QPEApplication::desktop()->height() )
									{
										bAnotherFullScreen = true;
									}
								}
								delete pObList;
																
								if ( bAnotherFullScreen )
								{
									DBG_LOG( QString( "save() > %1 widget" ).arg(aWin->caption()).latin1() );
									DBG_LOG( QString( "save() > widget height : %1" ).arg(aWin->height()).latin1() );
									DBG_LOG( "save() > ANOTHER FULL-SCREEN APP FOUND" );
									Global::execute( "launcher" );
									bRet = FALSE;
									m_nTempMode = LUS_NOSAVE;
									ResetIntervals();
									break;
								}
							}
						}
					}
					delete pList;
					if ( bAnotherFullScreen ) break;
#else
#if 0
					QPoint centerPos( QPEApplication::desktop()->width() / 2, QPEApplication::desktop()->height() / 2 );
					QWidget* pForeWidget = QApplication::widgetAt( centerPos, true );
					if ( pForeWidget && pForeWidget->height() == QPEApplication::desktop()->height() )
					{
						DBG_LOG( "save() > ANOTHER FULL-SCREEN APP FOUND" );
						Global::execute( "launcher" );
						bRet = FALSE;
						m_nTempMode = LUS_NOSAVE;
						ResetIntervals();
						break;
					}
#else
					if ( IsSpecialAppExists( LUS_NOSAVE ) )
					{
						DBG_LOG( "save() > ANOTHER FULL-SCREEN APP FOUND" );
						QTimer::singleShot( 10000, this, SLOT( ShowScreenSaver() ) );
						Global::execute( "launcher" );
						bRet = TRUE;
						m_nTempMode = LUS_SCREENSAVING;
						break;
					}
#endif
#endif
#endif // FIND Full Screen APP
//					QTimer::singleShot( 10, this, SLOT( ShowScreenSaver() ) );
					ShowScreenSaver();
					bRet = TRUE;
					m_nTempMode = LUS_SCREENSAVING;
					break;
				}
				else
				{
					DBG_LOG( "save() > Do Dimmer" );
					SetLightLevel( GetDimLevel() );
					bRet = true;
					m_nTempMode = LUS_SCREENSAVING;
					break;
				}
			}
		}
		DBG_LOG( "save() > judgement : up saving level to 1" );
	case 1:
		{
			if ( ( m_nDisableSuspend >= 0
				&& m_nDisableSuspend <= 1 )
				|| 
				IsSpecialAppExists( LUS_SCREENSAVING ) )
			{
				DBG_LOG( "save() > LEVEL1 intercepted" );
				bRet = true;
				break;
			}
			if ( WhichIsFirst( LUS_LIGHTOFF, LUS_SUSPEND ) == LUS_LIGHTOFF
					&& m_nTempMode != LUS_LIGHTOFF )
			{
				DBG_LOG( "save() > Do LightOff" );
				SetLightLevel( 0 );
				bRet = TRUE;
				m_nTempMode = LUS_LIGHTOFF;
				break;
			}
		}
		DBG_LOG( "save() > judgement : up saving level to 2" );
	case 2:
		{
			if ( ( m_nDisableSuspend >= 0
				&& m_nDisableSuspend <= 2 )
				|| 
				IsSpecialAppExists( LUS_SUSPEND ) )
			{
				DBG_LOG( "save() > LEVEL2 intercepted" );
				bRet = true;
				break;
			}
			{
				DBG_LOG( "save() > Do suspend" );
				m_nTempMode = LUS_NOSAVE;
				Global::execute( "suspend" );
				bRet = true;
				break;
			}
		}
	default:
		break;
	}
	return bRet;
}

void LUSScreenSaver::ShowScreenSaver()
{
	DBG_LOG( "LUSScreenSaver::ShowScreenSaver()" );

//	if ( m_bWaitingLauncher )
//	{
//		QTimer::singleShot( 10, this, SLOT( ShowScreenSaver() ) );
//	}

	if ( m_pSaver )
	{
		return;
	}
	
	QString sSaverName = GetSaverName();
	if ( sSaverName.isEmpty() )
	{
		return;
	}

	QString sSaverPath = AbsSaverPath( sSaverName );

	if ( !QFile::exists( sSaverPath ) )
	{
		ExecOKDlg( QString("\"%1\" Not Found" ).arg( sSaverName ) );
		return;
	}

	DBG_LOG( "LUSScreenSaver::ShowScreenSaver() > Module loading start" );
	m_pSaver = new QLibrary( sSaverPath, QLibrary::Immediately );

    LUSScreenSaverInterface* iface;
    if ( m_pSaver->queryInterface( IID_LUSScreenSaver, (QUnknownInterface**)&iface ) != QS_OK )
	{
		delete m_pSaver;
		m_pSaver = NULL;
		ExecOKDlg( QString("\"%1\" Loading Failure" ).arg( sSaverName ) );
		return;
	}

	m_pSaverWidget = iface->Init( LUSScreenSaverInterface::LUS_NORMAL );

	if ( m_pSaverWidget )
	{
		DBG_LOG( "LUSScreenSaver::ShowScreenSaver() > widget->showFullScreen()" );
		m_pSaverWidget->showFullScreen();
	}
	
	iface->Start();
	DBG_LOG( "LUSScreenSaver::ShowScreenSaver() > Module Start" );

	if ( ! IsAppLocked() )
	{
		PauseKeyHelper();
	}
	else
	{
		AuthDel();
	}

	return;
}

void LUSScreenSaver::SetForceExecFlag()
{
	DBG_LOG( "LUSScreenSaver::SetForceExecFlag()" );
	m_bForceExecuted = TRUE;
}

void LUSScreenSaver::SetScreenSaverMode( int nMode )
{
	DBG_LOG( "LUSScreenSaver::SetScreenSaverMode( int nMode )" );
	if ( nMode < 0 )
	{
		m_nDisableSuspend = 200;
		m_nDisableSuspendPre = 200;
	}
	else
	{
		if ( m_nDisableSuspendPre > nMode )
		{
			m_nDisableSuspend = nMode;
		}
		m_nDisableSuspendPre = nMode;
	}
}

int LUSScreenSaver::ExecOKDlg( const char* msg )
{
	return LUSScreenSaver::ExecBaseDlg(
					msg,
					QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape );
}

int LUSScreenSaver::ExecBaseDlg( const char* msg, int button1, int button2 )
{
	QMessageBox mb( "LUSScreenSaver",
					msg,
					QMessageBox::NoIcon,
					button1,
					button2, QMessageBox::NoButton, TaskBarWidget() );
	mb.setIconPixmap( Resource::loadPixmap( "LUSScreenSaver_about" ) );
	mb.adjustSize();

	QPoint oDSize( QPEApplication::desktop()->width(),
					QPEApplication::desktop()->height() );
	QPoint oASize( mb.width(), mb.height() );
	mb.move( ( oDSize -  oASize ) / 2 );

	return mb.exec();
}

void LUSScreenSaver::VictimDlg()
{
	QMessageBox* mb = new QMessageBox( "LUSScreenSaver",
					"LCD OFF\n( if false to off, prease close this )",
					QMessageBox::NoIcon,
					QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape,
					QMessageBox::NoButton, QMessageBox::NoButton, TaskBarWidget() );
	mb->setIconPixmap( Resource::loadPixmap( "LUSScreenSaver_about" ) );
	mb->adjustSize();

	QPoint oDSize( QPEApplication::desktop()->width(),
					QPEApplication::desktop()->height() );
	QPoint oASize( mb->width(), mb->height() );
	mb->move( ( oDSize -  oASize ) / 2 );

	mb->show();
}

void LUSScreenSaver::AuthDone( int nID )
{
	DBG_LOG( QString( "LUSScreenSaver::AuthDone( %1 )" ).arg( nID ).latin1() );

	RestartKeyHelper();
	
	if ( ! m_pAuthDlg ) 
	{
		DBG_LOG( "LUSScreenSaver::AuthDone() > There is no LUSAuth Dialog." );
		return;
	}

	LUSScreenSaverAuthInterface* iface_auth;
	if ( m_pAuthDlg->queryInterface( IID_LUSScreenSaver, (QUnknownInterface**)&iface_auth ) == QS_OK )
	{
		if ( iface_auth->Stop( nID ) )
		{
			iface_auth->release();
			m_pAuthDlg->unload();
			delete m_pAuthDlg;
			m_pAuthDlg = NULL;
			DBG_LOG( "LUSScreenSaver::AuthDone() > Authentication Done." );
		}
	}
	else
	{
		DBG_LOG( "LUSScreenSaver::AuthDone() > LUSAuth queryInterface() faild" );
		// FATAL ERROR 
		m_pAuthDlg->unload();
		delete m_pAuthDlg;
		m_pAuthDlg = NULL;
		RestartKeyHelper();
	}
}

void LUSScreenSaver::AuthDel()
{
	DBG_LOG( "LUSScreenSaver::AuthDel()" );

	if ( ! m_pAuthDlg ) 
	{
		DBG_LOG( "LUSScreenSaver::AuthDell() > There is no LUSAuth Dialog." );
		return;
	}

	LUSScreenSaverAuthInterface* iface_auth;
	if ( m_pAuthDlg->queryInterface( IID_LUSScreenSaver, (QUnknownInterface**)&iface_auth ) == QS_OK )
	{
		iface_auth->Delete();
		iface_auth->release();
	}
	m_pAuthDlg->unload();
	delete m_pAuthDlg;
	m_pAuthDlg = NULL;

	DBG_LOG( "LUSScreenSaver::AuthDel() > Done." );
}

void LUSScreenSaver::RestartKeyHelper()
{
	DBG_LOG( "LUSScreenSaver::RestartKeyHelper()" );
	UnsetKeyFilter();
	QCopEnvelope e( "QPE/KeyHelper", "restart()" );
}

void LUSScreenSaver::PauseKeyHelper()
{
	DBG_LOG( "LUSScreenSaver::PauseKeyHelper()" );
	QCopEnvelope* p_e = new QCopEnvelope( "QPE/KeyHelper", "pause()" );
	delete p_e;
	SetKeyFilter();
}

bool LUSScreenSaver::IsAppLocked()
{
	return (bool)m_pAuthDlg;
}

void LUSScreenSaver::DoLockedSuspend()
{
	if ( ! IsAppLocked() && IsLUSAuthInstalled() /* && IsPasswordEnable() */ )
	{
		DBG_LOG( "LUSScreenSaver::DoLockedSuspend() > Do Authentication" );
		LUSScreenSaverAuthInterface* iface_auth;
		m_pAuthDlg = new QLibrary( "lusauth", QLibrary::Immediately );
		if ( m_pAuthDlg->queryInterface( IID_LUSScreenSaver,
										(QUnknownInterface**)&iface_auth ) == QS_OK )
		{
			PauseKeyHelper();
			iface_auth->Start( rand(), TRUE );
			DBG_LOG( "LUSScreenSaver::DoLockedSuspend() > Do Suspend" );
			Global::execute( "suspend" );
		}
		else
		{
			DBG_LOG( "LUSScreenSaver::DoLockedSuspend() > LUSAuth quetyInterface() failed" );
			AuthDel();
			RestartKeyHelper();
		}
	}
}

void LUSScreenSaver::UnsetWaitingLauncherFlag()
{
	m_bWaitingLauncher = FALSE;
	m_nTempMode = LUS_SCREENSAVING;
	QTimer::singleShot( 10, this, SLOT( ShowScreenSaver() ) );
}

bool LUSScreenSaver::IsWaitingLauncher()
{
	return m_bWaitingLauncher;
}

void LUSScreenSaver::GetForeground()
{
	DBG_LOG( "LUSScreenSaver::GetForeground()" );
	if ( m_pSaverWidget )
	{
		DBG_LOG( "LUSScreenSaver::GetForeground() > widget->showFullScreen()" );
		m_pSaverWidget->showNormal();
		m_pSaverWidget->raise();
		m_pSaverWidget->showFullScreen();
	}
}

