/*
	Authenticate Module For LUSScreenSaver
	Copyright (C) 2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "../LUSScreenSaverMisc/LUSScreenSaverMisc.h"
#include "LUSScreenSaverPassInputDlg.h"
#include <qpe/qcopenvelope_qws.h>
#include <qapplication.h>
#include <qpe/config.h>
#include <sl/fepinterface.h>
#include <sl/jpnfeputil.h>
#include <unistd.h>

static const char* LUS_SALT =
#include "../LUSScreenSaver.salt"
;

using namespace LUSMisc;

LUSPassInputDlg::LUSPassInputDlg( int nID, QWidget* parent, const char* name )
	: QWidget( parent, name, WDestructiveClose )
{
	m_bAuth = 0;
	m_nID = nID;
	m_bFirstTest = TRUE;
	m_bSuspendMode = FALSE;

	if ( ! name )
	{
		setName( "LUSPassInputDlg" );
	}

	setBackgroundColor( QColor( 0, 0, 0) );
	erase();

	EmptyLayout = new QHBoxLayout( this, 3, 3 );

	BaseWidget = new QFrame( this, "BaseWidget" );
	
	int desktopX = QPEApplication::desktop()->width();
	int desktopY = QPEApplication::desktop()->height();

	BaseWidget->setFixedSize( desktopX / 3, desktopY / 3 );

	EmptyLayout->addWidget( BaseWidget );

	BaseLayout = new QVBoxLayout( BaseWidget );
	BaseLayout->setSpacing( 6 );
	BaseLayout->setMargin( 11 );
	
	ButtonsLayout = new QHBoxLayout( BaseWidget );
	ButtonsLayout->setSpacing( 6 );
	ButtonsLayout->setMargin( 11 );

	PassLabel = new QLabel( "Password", BaseWidget, "PassLabel" );
	PassEdit = new QLineEdit( BaseWidget, "PassEdit" );
	PassEdit->setEchoMode( QLineEdit::Password );

	OkButton = new QPushButton( BaseWidget, "OkButton" );
	OkButton->setText( "OK" );
	CancelButton = new QPushButton( BaseWidget, "CancelButton" );
	CancelButton->setText( "CANCEL" );

	ButtonsLayout->addWidget( OkButton );
	ButtonsLayout->addWidget( CancelButton );

	BaseLayout->addWidget( PassLabel );
	BaseLayout->addWidget( PassEdit );
	BaseLayout->addLayout( ButtonsLayout );

	m_pSysChannel = new QCopChannel( "QPE/System", this );
	connect( m_pSysChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveSysMessage( const QCString&, const QByteArray& ) ) );

	connect( OkButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( CancelButton, SIGNAL( clicked() ), this, SLOT( close() ) );

	PassEdit->setFocus();

	JpnFep::setInputMode( CIM_None | CZM_Hankaku, false );
	Global::hideInputMethod();
}

LUSPassInputDlg::~LUSPassInputDlg()
{
	DBG_LOG( "LUSPassInputDlg::~LUSPassInputDlg()" );
	delete m_pSysChannel;
}

void LUSPassInputDlg::resizeEvent( QResizeEvent*  )
{
	int desktopX = QPEApplication::desktop()->width();
	int desktopY = QPEApplication::desktop()->height();
	BaseWidget->setFixedSize( desktopX /3, desktopY / 3 );
	PassEdit->setFocus();
}

void LUSPassInputDlg::focusInEvent( QFocusEvent* )
{
	showNormal();
	raise();
	showFullScreen();
	PassEdit->setFocus();
}

void LUSPassInputDlg::accept()
{
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "Misc" );
	QString enc_pass = LUS_SALT;
	enc_pass += "$";
	QString conf_pass = cfg.readEntry( "Password", QString::null );
	enc_pass += conf_pass;

	QString inp_pass = crypt( PassEdit->text().latin1(), LUS_SALT );

	if ( ( conf_pass.isEmpty() ? PassEdit->text().isEmpty() : FALSE ) || enc_pass == inp_pass )
	{
		m_bAuth = TRUE;
		close();
	}
	else
	{
		m_bAuth = FALSE;
		PassLabel->setAlignment( AlignCenter | AlignVCenter | ExpandTabs );
		PassLabel->setText( "NO GOOD" );
		PassEdit->hide();
		OkButton->hide();
		CancelButton->hide();
		updateGeometry();
		QTimer::singleShot( 800, this, SLOT( close() ) );
	}
}

void LUSPassInputDlg::closeEvent( QCloseEvent* e )
{
	DBG_LOG( "LUSPassInputDlg::closeEvent()" );
	JpnFep::setInputMode( CIM_Hiragana | CZM_Zenkaku, false );
	Global::hideInputMethod();
	if ( ! m_bAuth )
	{
		if ( m_bSuspendMode )
		{
			QCopEnvelope* p_en = new QCopEnvelope( "QPE/LUSScreenSaver", "AuthDone(int)" );
			*p_en << m_nID;
			delete p_en;

			p_en = new QCopEnvelope( "QPE/LUSScreenSaver", "LockedSuspend()" );
			delete p_en;
		}
		else
		{
			QCopEnvelope en( "QPE/LUSScreenSaver", "SetScreenSaverActivate()" );
		}
	}
	else
	{
		QCopEnvelope en( "QPE/LUSScreenSaver", "AuthDone(int)" );
		en << m_nID;
	}
	e->ignore();
}

void LUSPassInputDlg::receiveSysMessage( const QCString& msg, const QByteArray& )
{
	if ( msg == "appForeground(QString)" || msg == "rotateChangedDone()" )
	{
		DBG_LOG( "LUSPassInputDlg::receiveSysMessage() > raise()" );
		showNormal();
		raise();
		showFullScreen();
		PassEdit->setFocus();
	}
}

void LUSPassInputDlg::keyPressEvent( QKeyEvent* e )
{
	if ( m_bFirstTest )
	{
		e->ignore();
		return;
	}

	if ( e->key() == Key_Return )
	{
		accept();
	}
	else if ( e->key() == Key_Escape )
	{
		close();
	}
}

void LUSPassInputDlg::keyReleaseEvent( QKeyEvent* e )
{
	if ( m_bFirstTest )
	{
		m_bFirstTest = FALSE;
		e->ignore();
		PassEdit->setText("");
	}
}

void LUSPassInputDlg::ResetID( int nID )
{
	m_nID = nID;
}

void LUSPassInputDlg::SetSuspendMode()
{
	m_bSuspendMode = TRUE;
}

